/*
 * DesktopApplication1View.java
 */

package cutandplay;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import java.io.*;
import javax.swing.JColorChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * The application's main frame.
 */
public class CutAndPlayView extends FrameView implements ToolPanelProvider {

    private SlideView slideview;
    private Presentation presentation;
    private ImageCropper cropper;
    private JPanel activeToolPanel;
    
    public CutAndPlayView(SingleFrameApplication app) {
        super(app);

        initComponents();

        /* setup slideview */
        //Slide slide = new Slide();
        slideview = new SlideView ();
        slideview.setSlide (null);
        slideview.toolPanelProvider = this;
        mainLayeredPane.add (slideview, new Integer (1));
        activeToolPanel = null;
        computeActiveMenuItems();
        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    @Action
    public void showFileFrame()
    {
        JFrame mainFrame = CutAndPlay.getApplication().getMainFrame();
        JFileChooser fc = new JFileChooser();
        int result = fc.showOpenDialog (mainFrame);
        if (result == fc.APPROVE_OPTION) {
	    SlideImage si = new SlideImage (fc.getSelectedFile());
	    if (si.load_error)
	    {
		JOptionPane.showMessageDialog (null, "Image file '" + fc.getSelectedFile()
			+ "' could not be loaded.", "Image load error", JOptionPane.ERROR_MESSAGE);
		slideview.slide.setBackgroundImage (null);
	    }
	    else
	    {
		slideview.slide.setBackgroundImage (si);
	    }
        }
        computeActiveMenuItems();
    }
    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = CutAndPlay.getApplication().getMainFrame();
            aboutBox = new CutAndPlayAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        CutAndPlay.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
        // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
        private void initComponents() {

                menuBar = new javax.swing.JMenuBar();
                javax.swing.JMenu fileMenu = new javax.swing.JMenu();
                newFileItem = new javax.swing.JMenuItem();
                openFileItem = new javax.swing.JMenuItem();
                saveFileItem = new javax.swing.JMenuItem();
                saveAsImageMenuItem = new javax.swing.JMenuItem();
                openImageMenuItem = new javax.swing.JMenuItem();
                cropImageMenuItem = new javax.swing.JMenuItem();
                addTextMenuItem = new javax.swing.JMenuItem();
                addBoxMenuItem = new javax.swing.JMenuItem();
                javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
                editMenu = new javax.swing.JMenu();
                deleteMenuItem = new javax.swing.JMenuItem();
                javax.swing.JMenu helpMenu = new javax.swing.JMenu();
                javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
                statusPanel = new javax.swing.JPanel();
                javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
                statusMessageLabel = new javax.swing.JLabel();
                statusAnimationLabel = new javax.swing.JLabel();
                progressBar = new javax.swing.JProgressBar();
                mainLayeredPane = new javax.swing.JLayeredPane();
                cropToolPanel = new javax.swing.JPanel();
                jButton1 = new javax.swing.JButton();
                jButton2 = new javax.swing.JButton();
                textToolPanel = new javax.swing.JPanel();
                fontSizeComboBox = new javax.swing.JComboBox();
                jLabel1 = new javax.swing.JLabel();
                jLabel2 = new javax.swing.JLabel();
                jButton3 = new javax.swing.JButton();
                boxToolPanel = new javax.swing.JPanel();
                jLabel3 = new javax.swing.JLabel();
                jButton4 = new javax.swing.JButton();

                menuBar.setName("menuBar"); // NOI18N

                javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(cutandplay.CutAndPlay.class).getContext().getActionMap(CutAndPlayView.class, this);
                fileMenu.setAction(actionMap.get("loadFile")); // NOI18N
                org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(cutandplay.CutAndPlay.class).getContext().getResourceMap(CutAndPlayView.class);
                fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
                fileMenu.setName("fileMenu"); // NOI18N

                newFileItem.setAction(actionMap.get("newFile")); // NOI18N
                newFileItem.setText(resourceMap.getString("newFileItem.text")); // NOI18N
                newFileItem.setName("newFileItem"); // NOI18N
                fileMenu.add(newFileItem);

                openFileItem.setAction(actionMap.get("loadFile")); // NOI18N
                openFileItem.setText(resourceMap.getString("openFileItem.text")); // NOI18N
                openFileItem.setName("openFileItem"); // NOI18N
                fileMenu.add(openFileItem);

                saveFileItem.setAction(actionMap.get("saveFile")); // NOI18N
                saveFileItem.setText(resourceMap.getString("saveFileItem.text")); // NOI18N
                saveFileItem.setName("saveFileItem"); // NOI18N
                fileMenu.add(saveFileItem);

                saveAsImageMenuItem.setAction(actionMap.get("saveAsImage")); // NOI18N
                saveAsImageMenuItem.setText(resourceMap.getString("saveAsImageMenuItem.text")); // NOI18N
                saveAsImageMenuItem.setName("saveAsImageMenuItem"); // NOI18N
                fileMenu.add(saveAsImageMenuItem);

                openImageMenuItem.setAction(actionMap.get("showFileFrame")); // NOI18N
                openImageMenuItem.setText(resourceMap.getString("openImageMenuItem.text")); // NOI18N
                openImageMenuItem.setName("openImageMenuItem"); // NOI18N
                fileMenu.add(openImageMenuItem);

                cropImageMenuItem.setAction(actionMap.get("cropImage")); // NOI18N
                cropImageMenuItem.setText(resourceMap.getString("cropImageMenuItem.text")); // NOI18N
                cropImageMenuItem.setName("cropImageMenuItem"); // NOI18N
                fileMenu.add(cropImageMenuItem);

                addTextMenuItem.setAction(actionMap.get("addText")); // NOI18N
                addTextMenuItem.setText(resourceMap.getString("addTextMenuItem.text")); // NOI18N
                addTextMenuItem.setName("addTextMenuItem"); // NOI18N
                fileMenu.add(addTextMenuItem);

                addBoxMenuItem.setAction(actionMap.get("addBox")); // NOI18N
                addBoxMenuItem.setText(resourceMap.getString("addBoxMenuItem.text")); // NOI18N
                addBoxMenuItem.setName("addBoxMenuItem"); // NOI18N
                fileMenu.add(addBoxMenuItem);

                exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
                exitMenuItem.setName("exitMenuItem"); // NOI18N
                fileMenu.add(exitMenuItem);

                menuBar.add(fileMenu);

                editMenu.setText(resourceMap.getString("editMenu.text")); // NOI18N
                editMenu.setName("editMenu"); // NOI18N

                deleteMenuItem.setAction(actionMap.get("deleteSelected")); // NOI18N
                deleteMenuItem.setText(resourceMap.getString("deleteMenuItem.text")); // NOI18N
                deleteMenuItem.setName("deleteMenuItem"); // NOI18N
                editMenu.add(deleteMenuItem);

                menuBar.add(editMenu);

                helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
                helpMenu.setName("helpMenu"); // NOI18N

                aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
                aboutMenuItem.setName("aboutMenuItem"); // NOI18N
                helpMenu.add(aboutMenuItem);

                menuBar.add(helpMenu);

                statusPanel.setName("statusPanel"); // NOI18N

                statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

                statusMessageLabel.setName("statusMessageLabel"); // NOI18N

                statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
                statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

                progressBar.setName("progressBar"); // NOI18N

                javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
                statusPanel.setLayout(statusPanelLayout);
                statusPanelLayout.setHorizontalGroup(
                        statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 524, Short.MAX_VALUE)
                        .addGroup(statusPanelLayout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(statusMessageLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 340, Short.MAX_VALUE)
                                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(statusAnimationLabel)
                                .addContainerGap())
                );
                statusPanelLayout.setVerticalGroup(
                        statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(statusPanelLayout.createSequentialGroup()
                                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(statusMessageLabel)
                                        .addComponent(statusAnimationLabel)
                                        .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(3, 3, 3))
                );

                mainLayeredPane.setBackground(resourceMap.getColor("mainLayeredPane.background")); // NOI18N
                mainLayeredPane.setName("mainLayeredPane"); // NOI18N
                mainLayeredPane.setOpaque(true);
                mainLayeredPane.addComponentListener(new java.awt.event.ComponentAdapter() {
                        public void componentResized(java.awt.event.ComponentEvent evt) {
                                mainLayeredPaneComponentResized(evt);
                        }
                });

                cropToolPanel.setName("cropToolPanel"); // NOI18N

                jButton1.setAction(actionMap.get("executeCrop")); // NOI18N
                jButton1.setText(resourceMap.getString("jCropButton.text")); // NOI18N
                jButton1.setName("jCropButton"); // NOI18N

                jButton2.setAction(actionMap.get("cancelCrop")); // NOI18N
                jButton2.setText(resourceMap.getString("jButton2.text")); // NOI18N
                jButton2.setName("jButton2"); // NOI18N

                javax.swing.GroupLayout cropToolPanelLayout = new javax.swing.GroupLayout(cropToolPanel);
                cropToolPanel.setLayout(cropToolPanelLayout);
                cropToolPanelLayout.setHorizontalGroup(
                        cropToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, cropToolPanelLayout.createSequentialGroup()
                                .addGap(31, 31, 31)
                                .addComponent(jButton2)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 84, Short.MAX_VALUE)
                                .addComponent(jButton1)
                                .addGap(30, 30, 30))
                );
                cropToolPanelLayout.setVerticalGroup(
                        cropToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, cropToolPanelLayout.createSequentialGroup()
                                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addGroup(cropToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(jButton1)
                                        .addComponent(jButton2))
                                .addGap(34, 34, 34))
                );

                cropToolPanel.setBounds(80, 210, 330, 50);
                mainLayeredPane.add(cropToolPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

                textToolPanel.setName("textToolPanel"); // NOI18N

                fontSizeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "20", "25", "30", "35", "40", "50", "60", "70", "80", "90", "100", "120", "140", "160" }));
                fontSizeComboBox.setName("fontSizeComboBox"); // NOI18N
                fontSizeComboBox.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                fontSizeComboBoxActionPerformed(evt);
                        }
                });

                jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
                jLabel1.setName("jLabel1"); // NOI18N

                jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
                jLabel2.setName("jLabel2"); // NOI18N

                jButton3.setAction(actionMap.get("chooseTextColor")); // NOI18N
                jButton3.setText(resourceMap.getString("jButton3.text")); // NOI18N
                jButton3.setName("jButton3"); // NOI18N

                javax.swing.GroupLayout textToolPanelLayout = new javax.swing.GroupLayout(textToolPanel);
                textToolPanel.setLayout(textToolPanelLayout);
                textToolPanelLayout.setHorizontalGroup(
                        textToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(textToolPanelLayout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(jLabel1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(fontSizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jLabel2)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jButton3, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addContainerGap(30, Short.MAX_VALUE))
                );
                textToolPanelLayout.setVerticalGroup(
                        textToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(textToolPanelLayout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(textToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(fontSizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(jLabel1)
                                        .addComponent(jButton3)
                                        .addComponent(jLabel2))
                                .addContainerGap(13, Short.MAX_VALUE))
                );

                textToolPanel.setBounds(80, 70, -1, 50);
                mainLayeredPane.add(textToolPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

                boxToolPanel.setName("boxToolPanel"); // NOI18N

                jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
                jLabel3.setName("jLabel3"); // NOI18N

                jButton4.setAction(actionMap.get("chooseFillColor")); // NOI18N
                jButton4.setText(resourceMap.getString("jButton4.text")); // NOI18N
                jButton4.setName("jButton4"); // NOI18N

                javax.swing.GroupLayout boxToolPanelLayout = new javax.swing.GroupLayout(boxToolPanel);
                boxToolPanel.setLayout(boxToolPanelLayout);
                boxToolPanelLayout.setHorizontalGroup(
                        boxToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(boxToolPanelLayout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(jLabel3)
                                .addGap(18, 18, 18)
                                .addComponent(jButton4)
                                .addContainerGap(175, Short.MAX_VALUE))
                );
                boxToolPanelLayout.setVerticalGroup(
                        boxToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(boxToolPanelLayout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(boxToolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(jButton4)
                                        .addComponent(jLabel3))
                                .addContainerGap(13, Short.MAX_VALUE))
                );

                boxToolPanel.setBounds(80, 140, -1, 50);
                mainLayeredPane.add(boxToolPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

                setComponent(mainLayeredPane);
                setMenuBar(menuBar);
                setStatusBar(statusPanel);
        }// </editor-fold>//GEN-END:initComponents

    private void mainLayeredPaneComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_mainLayeredPaneComponentResized
       reLayoutMainLayeredPane();
    }//GEN-LAST:event_mainLayeredPaneComponentResized

    private void reLayoutMainLayeredPane () {
        int width, height;
        if (activeToolPanel == null)
        {
            width = mainLayeredPane.getWidth();
            height = mainLayeredPane.getHeight();
        }
        else
        {
            activeToolPanel.setBounds (0, mainLayeredPane.getHeight() - 50,
                                          mainLayeredPane.getWidth(), 50);
            width = mainLayeredPane.getWidth();
            height = mainLayeredPane.getHeight() - 50;
        }
        slideview.setBounds (0, 0, width, height);
        if (cropper != null)
            cropper.setBounds (0, 0, width, height);
        computeActiveMenuItems();
    }
    private void fontSizeComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fontSizeComboBoxActionPerformed
        selectedText.setFontSize (4 * Integer.parseInt (fontSizeComboBox.getSelectedItem().toString()));
}//GEN-LAST:event_fontSizeComboBoxActionPerformed

    @Action
    public void cropImage() {
        if (slideview.slide.backgroundImage == null)
        {
            JOptionPane.showMessageDialog(null, "Kein Hintergrundbild", "Fehler", JOptionPane.OK_CANCEL_OPTION);
        }
        else
        {
            hidePanels();
            cropper = new ImageCropper (slideview.slide.backgroundImage.image,
	                                presentation.aspect_x, presentation.aspect_y);
            cropToolPanel.setVisible (true);
            mainLayeredPane.add (cropper, new Integer (3));
            mainLayeredPane.add (cropToolPanel, new Integer (4));
            mainLayeredPane.setLayer (cropToolPanel, new Integer (4));
            activeToolPanel = cropToolPanel;
            reLayoutMainLayeredPane();
        }
    }

    @Action
    public void executeCrop() {
        slideview.slide.backgroundImage.area = cropper.getCropArea();
        slideview.redraw();
        cancelCrop();
    }
    public void cancelCrop() {
        mainLayeredPane.remove (cropper);
        mainLayeredPane.remove (cropToolPanel);
        cropper = null;
        activeToolPanel = null;
        reLayoutMainLayeredPane();
    }

    @Action
    public void addText() {
	Text text = new Text (slideview.slide);
        slideview.slide.addElement (text);
	slideview.setFocus (text);
    }
    @Action
    public void addBox() {
	Box box = new Box (slideview.slide);
        slideview.slide.addElement (box);
	slideview.setFocus (box);
    }
        // Variables declaration - do not modify//GEN-BEGIN:variables
        private javax.swing.JMenuItem addBoxMenuItem;
        private javax.swing.JMenuItem addTextMenuItem;
        private javax.swing.JPanel boxToolPanel;
        private javax.swing.JMenuItem cropImageMenuItem;
        private javax.swing.JPanel cropToolPanel;
        private javax.swing.JMenuItem deleteMenuItem;
        private javax.swing.JMenu editMenu;
        private javax.swing.JComboBox fontSizeComboBox;
        private javax.swing.JButton jButton1;
        private javax.swing.JButton jButton2;
        private javax.swing.JButton jButton3;
        private javax.swing.JButton jButton4;
        private javax.swing.JLabel jLabel1;
        private javax.swing.JLabel jLabel2;
        private javax.swing.JLabel jLabel3;
        private javax.swing.JLayeredPane mainLayeredPane;
        private javax.swing.JMenuBar menuBar;
        private javax.swing.JMenuItem newFileItem;
        private javax.swing.JMenuItem openFileItem;
        private javax.swing.JMenuItem openImageMenuItem;
        private javax.swing.JProgressBar progressBar;
        private javax.swing.JMenuItem saveAsImageMenuItem;
        private javax.swing.JMenuItem saveFileItem;
        private javax.swing.JLabel statusAnimationLabel;
        private javax.swing.JLabel statusMessageLabel;
        private javax.swing.JPanel statusPanel;
        private javax.swing.JPanel textToolPanel;
        // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
    private JFrame fileFrame;

    private Text selectedText;
    private Box selectedBox;

    public void showTextPanel (Text text) {
        hidePanels();
        mainLayeredPane.add (textToolPanel, new Integer (4));
        mainLayeredPane.setLayer (textToolPanel, new Integer (4));
        activeToolPanel = textToolPanel;
        activeToolPanel.setVisible (true);
        selectedText = text;
        Object item;
        for (int i = 0; (item = fontSizeComboBox.getItemAt(i)) != null; i++)
        {
            if (item.toString().equals (new Integer (text.getFontSize() / 4).toString()))
                fontSizeComboBox.setSelectedItem (item);
        }
        //System.out.println ("TextPanel activated");
        reLayoutMainLayeredPane();
    }

    public void showBoxPanel (Box box) {
        hidePanels();
        mainLayeredPane.add (boxToolPanel, new Integer (4));
        mainLayeredPane.setLayer (boxToolPanel, new Integer (4));
        activeToolPanel = boxToolPanel;
        activeToolPanel.setVisible (true);
        selectedBox = box;
        reLayoutMainLayeredPane();
    }

    public void showCropPanel() {
        hidePanels();
        activeToolPanel = cropToolPanel;
        reLayoutMainLayeredPane();
    }

    public void hidePanels() {
        boxToolPanel.setVisible (false);
        textToolPanel.setVisible (false);
        cropToolPanel.setVisible (false);
        activeToolPanel = null;
        reLayoutMainLayeredPane();
    }

    @Action
    public void chooseFillColor() {
        Color color = JColorChooser.showDialog (mainLayeredPane, "Choose Fill Color", selectedBox.getBgColor());
        if (color != null)
            selectedBox.setBgColor (color);
    }

    @Action
    public void chooseTextColor() {
        Color color = JColorChooser.showDialog (mainLayeredPane, "Choose Fill Color", selectedText.getColor());
        if (color != null)
            selectedText.setColor (color);
    }
    @Action
    public void newFile() {
        JFrame mainFrame = CutAndPlay.getApplication().getMainFrame();
        NewPresentationDialog dialog = new NewPresentationDialog (mainFrame, true);
	dialog.setVisible (true);
	if (dialog.aspect_x > 0 && dialog.aspect_y > 0)
	{
	    presentation = new Presentation (dialog.aspect_x, dialog.aspect_y);
	    Slide slide = presentation.addSlide();
	    slideview.setSlide (slide);
	}
        computeActiveMenuItems();
    }

    @Action
    public void loadFile() {
        JFrame mainFrame = CutAndPlay.getApplication().getMainFrame();
        JFileChooser fc = new JFileChooser();
        int result = fc.showOpenDialog (mainFrame);
        if (result == fc.APPROVE_OPTION) {
	    presentation = new Presentation (4, 3); /* aspect will be overwritten from file aspect */
            if (presentation.load (fc.getSelectedFile()))
	    {
		slideview.setSlide ((Slide) presentation.slides.firstElement());
	    }
	    else
	    {
		JOptionPane.showMessageDialog(null, "The file '" + fc.getSelectedFile()
			+ "' could not be loaded", "File load error", JOptionPane.ERROR_MESSAGE);
		presentation = null;
		slideview.setSlide (null);
		slideview.redraw();
	    }
        }
        computeActiveMenuItems();
    }

    @Action
    public void saveFile() {
        JFrame mainFrame = CutAndPlay.getApplication().getMainFrame();
        JFileChooser fc = new JFileChooser();
        int result = fc.showSaveDialog (mainFrame);
        if (result == fc.APPROVE_OPTION) {
            if (!presentation.save (fc.getSelectedFile()))
	    {
		JOptionPane.showMessageDialog (null, "The file '"+fc.getSelectedFile()
			+ "' could not be saved", "File save error", JOptionPane.ERROR_MESSAGE);
	    }
        }
        computeActiveMenuItems();
    }

    private void computeActiveMenuItems() {
        boolean inCrop = (activeToolPanel == cropToolPanel);
	boolean haveSlide = slideview.slide != null;
	newFileItem.setEnabled (!inCrop);
	openFileItem.setEnabled (!inCrop);
	saveFileItem.setEnabled (haveSlide && !inCrop);
	saveAsImageMenuItem.setEnabled (haveSlide && !inCrop);
        openImageMenuItem.setEnabled (haveSlide && !inCrop);
        cropImageMenuItem.setEnabled (haveSlide && slideview.slide.backgroundImage != null && !inCrop);
        addTextMenuItem.setEnabled (haveSlide && !inCrop);
        addBoxMenuItem.setEnabled (haveSlide && !inCrop);
	deleteMenuItem.setEnabled (haveSlide && !inCrop);
    }

    @Action
    public void deleteSelected()
    {
	slideview.slide.deleteSelected();
    }

    @Action
    public void saveAsImage()
    {
	JFrame mainFrame = CutAndPlay.getApplication().getMainFrame();
	JFileChooser fc = new JFileChooser();
	int result = fc.showSaveDialog (mainFrame);
	if (result == fc.APPROVE_OPTION)
	{
	    Slide slide = (Slide) presentation.slides.firstElement();
	    ImageResolutionDialog ird = new ImageResolutionDialog (mainFrame, true,
		                                                   presentation.aspect_x,
								   presentation.aspect_y);
	    ird.setVisible (true);
	    int w = ird.getW();
	    int h = ird.getH();
	    BufferedImage image = new BufferedImage (w, h, BufferedImage.TYPE_INT_ARGB);
	    Graphics2D g2 = image.createGraphics();
	    slide.draw(g2, new Rectangle (0, 0, w, h));
	    try {
		ImageIO.write(image, "png", fc.getSelectedFile());
	    } catch (Exception ex) {
		JOptionPane.showMessageDialog(null, "The file '" + fc.getSelectedFile() + "' could not be saved", "File save error", JOptionPane.ERROR_MESSAGE);
	    }
	}
	computeActiveMenuItems();
    }
}
