/*
 * This file is part of CutAndPlay.
 *
 * Copyright (C) 2009 Stefan Westerfeld
 * Copyright (C) 2009 Treuwert Computer GmbH
 *
 * CutAndPlay is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package cutandplay;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.Collections;
import java.util.Comparator;
import java.util.ListIterator;
import java.util.Vector;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author stefan
 */
public class Slide {
    public SlideImage backgroundImage;
    public SlideView view;
    public Presentation presentation;
    public Vector elements;

    Slide (Presentation presentation) {
        view = null;
        backgroundImage = null;
	this.presentation = presentation;
        elements = new Vector();
    }

    void deleteSelected() {
	Vector newElements = new Vector();
	ListIterator iter = elements.listIterator();
	while (iter.hasNext())
	{
	    SlideElement element = (SlideElement) iter.next();
	    if (!element.selected)
		newElements.add (element);
	}
	elements = newElements;
	if (view != null)
	{
	    view.setFocus(null);
	    view.redraw();
	}
    }

    void setBackgroundImage (SlideImage backgroundImage)
    {
        this.backgroundImage = backgroundImage;
        if (view != null)
            view.redraw();
    }
    void addElement (SlideElement element)
    {
        elements.add (element);
        Collections.sort(elements, new Comparator() {
            public int compare(Object o1, Object o2) {
                SlideElement e1 = (SlideElement) o1;
                SlideElement e2 = (SlideElement) o2;
                return e1.layer() - e2.layer();
            }
        });
        if (view != null)
            view.redraw();
    }
    void load (XMLStreamReader r) throws XMLStreamException
    {
        while (r.hasNext())
        {
            int event = r.next();
            switch (event)
            {
                case XMLStreamConstants.START_ELEMENT:
                    {
                        SlideElement element = null;

                        if (r.getLocalName().equals ("box"))
                            element = new Box (this);
                        if (r.getLocalName().equals ("text"))
                            element = new Text (this);
                        if (r.getLocalName().equals ("background_image"))
                        {
                            backgroundImage = new SlideImage ();
                            backgroundImage.load (r);
                        }
                        if (element != null)
                        {
                            element.load (r);
                            elements.add (element);
                        }
                    }
                    break;
                case XMLStreamConstants.END_ELEMENT:
                    if (r.getLocalName().equals ("slide"))
                        return;
                    break;
            }
        }
        throw new XMLStreamException("expected end of slide element before EOF");
    }
    void save (XMLStreamWriter xtw) throws XMLStreamException
    {
	xtw.writeStartElement("slide");
	xtw.writeCharacters("\n");
	if (backgroundImage != null)
	{
	    backgroundImage.save(xtw);
	    xtw.writeCharacters("\n");
	}
	ListIterator iter = elements.listIterator();
	while (iter.hasNext()) {
	    SlideElement element = (SlideElement) iter.next();
	    element.save(xtw);
	    xtw.writeCharacters("\n");
	}
	xtw.writeEndElement();
    }

    void draw (Graphics2D g2, Rectangle drect)
    {
	g2.setPaint(Color.black);
	g2.fill(drect);
	if (backgroundImage != null)
	    backgroundImage.draw(g2, drect);
	ListIterator iter = elements.listIterator();
	while (iter.hasNext()) {
	    SlideElement element = (SlideElement) iter.next();
	    element.draw(g2, drect);
	}
    }
}
